@@ -328,7 +328,7 @@ gtk_theme_changed (GConfPropertyEditor *peditor,
   }
 
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "gtk_themes_delete"),
-			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_REGULAR));
+			    theme_is_writable (theme));
 }
 
 static void
@@ -344,7 +344,7 @@ window_theme_changed (GConfPropertyEditor *peditor,
     theme = gnome_theme_info_find (name);
 
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "window_themes_delete"),
-			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_REGULAR));
+			    theme_is_writable (theme));
 }
 
 static void
@@ -360,19 +360,49 @@ icon_theme_changed (GConfPropertyEditor *peditor,
     theme = gnome_theme_icon_info_find (name);
 
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "icon_themes_delete"),
-			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_ICON));
+			    theme_is_writable (theme));
 }
 
 #ifdef HAVE_XCURSOR
+static void
+cursor_size_changed_cb (int size, AppearanceData *data)
+{
+  gconf_client_set_int (data->client, CURSOR_SIZE_KEY, size, NULL);
+}
+
+static void
+cursor_size_scale_value_changed_cb (GtkRange *range, AppearanceData *data)
+{
+  GnomeThemeCursorInfo *theme;
+  gchar *name;
+
+  name = gconf_client_get_string (data->client, CURSOR_THEME_KEY, NULL);
+  if (name == NULL)
+    return;
+
+  theme = gnome_theme_cursor_info_find (name);
+  g_free (name);
+
+  if (theme) {
+    gint size;
+
+    size = g_array_index (theme->sizes, gint, (int) gtk_range_get_value (range));
+    cursor_size_changed_cb (size, data);
+  }
+}
+#endif
+
 static void
 update_cursor_size_scale (GnomeThemeCursorInfo *theme,
                           AppearanceData *data)
 {
+#ifdef HAVE_XCURSOR
   GtkWidget *cursor_size_scale;
   GtkWidget *cursor_size_label;
   GtkWidget *cursor_size_small_label;
   GtkWidget *cursor_size_large_label;
   gboolean sensitive;
+  gint size, gconf_size;
 
   cursor_size_scale = glade_xml_get_widget (data->xml, "cursor_size_scale");
   cursor_size_label = glade_xml_get_widget (data->xml, "cursor_size_label");
@@ -385,59 +415,57 @@ update_cursor_size_scale (GnomeThemeCursorInfo *theme,
   gtk_widget_set_sensitive (cursor_size_small_label, sensitive);
   gtk_widget_set_sensitive (cursor_size_large_label, sensitive);
 
-  if (sensitive)
-  {
+  gconf_size = gconf_client_get_int (data->client, CURSOR_SIZE_KEY, NULL);
+
+  if (sensitive) {
     GtkAdjustment *adjustment;
-    gint gconf_size, i;
-    gboolean size_found = FALSE;
+    gint i, index;
     GtkRange *range = GTK_RANGE (cursor_size_scale);
 
     adjustment = gtk_range_get_adjustment (range);
     g_object_set (adjustment, "upper", (gdouble) theme->sizes->len - 1, NULL);
 
-    gconf_size = gconf_client_get_int (data->client, CURSOR_SIZE_KEY, NULL);
+
+    /* fallback if the gconf value is bigger than all available sizes;
+       use the largest we have */
+    index = theme->sizes->len - 1;
 
     /* set the slider to the cursor size which matches the gconf setting best  */
-    for (i = 0; i < theme->sizes->len; i++)
-    {
-      gint size;
+    for (i = 0; i < theme->sizes->len; i++) {
       size = g_array_index (theme->sizes, gint, i);
 
-      if (size == gconf_size)
-      {
-        gtk_range_set_value (range, (gdouble) i);
-        size_found = TRUE;
+      if (size == gconf_size) {
+      	index = i;
         break;
-      }
-      else if (size > gconf_size)
-      {
-        if (i == 0)
-        {
-          gtk_range_set_value (range, 0);
-          size_found = TRUE;
-          break;
-        }
-        else
-        {
+      } else if (size > gconf_size) {
+        if (i == 0) {
+          index = 0;
+        } else {
           gint diff, diff_to_last;
 
           diff = size - gconf_size;
           diff_to_last = gconf_size - g_array_index (theme->sizes, gint, i - 1);
 
-          gtk_range_set_value (range, (gdouble) (diff < diff_to_last) ? diff : diff_to_last);
-          size_found = TRUE;
-          break;
+          index = (diff < diff_to_last) ? i : i - 1;
         }
+        break;
       }
     }
 
-    /* set to the biggest size if the gconf value is bigger than
-      all available sizes */
-    if (!size_found)
-      gtk_range_set_value (range, (gdouble) g_array_index (theme->sizes, gint, theme->sizes->len - 1));
+    gtk_range_set_value (range, (gdouble) index);
+
+    size = g_array_index (theme->sizes, gint, index);
+  } else {
+    if (theme && theme->sizes->len > 0)
+      size = g_array_index (theme->sizes, gint, 0);
+    else
+      size = 18;
   }
-}
+
+  if (size != gconf_size)
+    cursor_size_changed_cb (size, data);
 #endif
+}
 
 static void
 cursor_theme_changed (GConfPropertyEditor *peditor,
@@ -451,12 +479,10 @@ cursor_theme_changed (GConfPropertyEditor *peditor,
   if (value && (name = gconf_value_get_string (value)))
     theme = gnome_theme_cursor_info_find (name);
 
-#ifdef HAVE_XCURSOR
   update_cursor_size_scale (theme, data);
-#endif
 
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "cursor_themes_delete"),
-			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_CURSOR));
+			    theme_is_writable (theme));
 
 }
 
@@ -515,28 +541,11 @@ icon_theme_delete_cb (GtkWidget *button, AppearanceData *data)
   generic_theme_delete ("icon_themes_list", THEME_TYPE_ICON, data);
 }
 
-#ifdef HAVE_XCURSOR
 static void
-cursor_size_scale_value_changed_cb (GtkRange *range, AppearanceData *data)
+cursor_theme_delete_cb (GtkWidget *button, AppearanceData *data)
 {
-  GnomeThemeCursorInfo *theme;
-  gchar *name;
-
-  name = gconf_client_get_string (data->client, CURSOR_THEME_KEY, NULL);
-  if (name == NULL)
-    return;
-
-  theme = gnome_theme_cursor_info_find (name);
-  g_free (name);
-
-  if (theme) {
-    gint size;
-
-    size = g_array_index (theme->sizes, gint, (int) gtk_range_get_value (range));
-    gconf_client_set_int (data->client, CURSOR_SIZE_KEY, size, NULL);
-  }
+  generic_theme_delete ("cursor_themes_list", THEME_TYPE_CURSOR, data);
 }
-#endif
 
 static void
 add_to_treeview (const gchar *tv_name,
@@ -653,14 +662,13 @@ icon_theme_thumbnail_cb (GdkPixbuf *pixbuf,
 }
 
 static void
-changed_on_disk_cb (GnomeThemeType       type,
-		    gpointer             theme,
-		    GnomeThemeChangeType change_type,
-		    GnomeThemeElement    element,
-		    AppearanceData      *data)
+changed_on_disk_cb (GnomeThemeCommonInfo *theme,
+		    GnomeThemeChangeType  change_type,
+		    GnomeThemeElement     element,
+		    AppearanceData       *data)
 {
-  if (type == GNOME_THEME_TYPE_REGULAR) {
-    GnomeThemeInfo *info = theme;
+  if (theme->type == GNOME_THEME_TYPE_REGULAR) {
+    GnomeThemeInfo *info = (GnomeThemeInfo *) theme;
 
     if (change_type == GNOME_THEME_CHANGE_DELETED) {
       if (info->has_gtk)
@@ -690,8 +698,8 @@ changed_on_disk_cb (GnomeThemeType       type,
       }
     }
 
-  } else if (type == GNOME_THEME_TYPE_ICON) {
-    GnomeThemeIconInfo *info = theme;
+  } else if (theme->type == GNOME_THEME_TYPE_ICON) {
+    GnomeThemeIconInfo *info = (GnomeThemeIconInfo *) theme;
 
     if (change_type == GNOME_THEME_CHANGE_DELETED) {
       remove_from_treeview ("icon_themes_list", info->name, data);
@@ -704,6 +712,18 @@ changed_on_disk_cb (GnomeThemeType       type,
       generate_icon_theme_thumbnail_async (info,
           (ThemeThumbnailFunc) icon_theme_thumbnail_cb, data, NULL);
     }
+
+  } else if (theme->type == GNOME_THEME_TYPE_CURSOR) {
+    GnomeThemeCursorInfo *info = (GnomeThemeCursorInfo *) theme;
+
+    if (change_type == GNOME_THEME_CHANGE_DELETED) {
+      remove_from_treeview ("cursor_themes_list", info->name, data);
+    } else {
+      if (change_type == GNOME_THEME_CHANGE_CREATED)
+        add_to_treeview ("cursor_themes_list", info->name, info->readable_name, info->thumbnail, data);
+      else if (change_type == GNOME_THEME_CHANGE_CHANGED)
+        update_in_treeview ("cursor_themes_list", info->name, info->readable_name, data);
+    }
   }
 }
 
@@ -751,11 +771,7 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
     case THEME_TYPE_CURSOR:
       themes = gnome_theme_cursor_info_find_all ();
       thumbnail = NULL;
-#ifdef HAVE_XCURSOR
       key = CURSOR_THEME_KEY;
-#else
-      key = CURSOR_FONT_KEY;
-#endif
       generator = NULL;
       thumb_cb = NULL;
       break;
@@ -769,32 +785,18 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
 
   for (l = themes; l; l = g_list_next (l))
   {
-    const gchar *name = NULL;
-    const gchar *label = NULL;
+    GnomeThemeCommonInfo *theme = (GnomeThemeCommonInfo *) l->data;
     GtkTreeIter i;
 
-    if (type == THEME_TYPE_GTK || type == THEME_TYPE_WINDOW) {
-      name = ((GnomeThemeInfo *) l->data)->name;
-    } else if (type == THEME_TYPE_ICON) {
-      name = ((GnomeThemeIconInfo *) l->data)->name;
-      label = ((GnomeThemeIconInfo *) l->data)->readable_name;
-    } else if (type == THEME_TYPE_CURSOR) {
-      name = ((GnomeThemeCursorInfo *) l->data)->name;
-      label = ((GnomeThemeCursorInfo *) l->data)->readable_name;
-    }
-
-    if (!name)
-      continue; /* just in case... */
-
     if (type == THEME_TYPE_CURSOR) {
-      thumbnail = ((GnomeThemeCursorInfo *) l->data)->thumbnail;
+      thumbnail = ((GnomeThemeCursorInfo *) theme)->thumbnail;
     } else {
-      generator (l->data, thumb_cb, data, NULL);
+      generator (theme, thumb_cb, data, NULL);
     }
 
     gtk_list_store_insert_with_values (store, &i, 0,
-                                       COL_LABEL, label ? label : name,
-                                       COL_NAME, name,
+                                       COL_LABEL, theme->readable_name,
+                                       COL_NAME, theme->name,
                                        COL_THUMBNAIL, thumbnail,
                                        -1);
 
@@ -805,17 +807,6 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
   }
   g_list_free (themes);
 
-#ifdef HAVE_XCURSOR
-  if (type == THEME_TYPE_CURSOR && !gnome_theme_cursor_info_find ("default")) {
-    GtkTreeIter i;
-    gtk_list_store_insert_with_values (store, &i, 0,
-                                       COL_LABEL, _("Default Pointer"),
-                                       COL_NAME, "default",
-                                       COL_THUMBNAIL, NULL,
-                                       -1);
-  }
-#endif
-
   sort_model = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (store));
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (sort_model),
                                         COL_LABEL, GTK_SORT_ASCENDING);
@@ -861,7 +852,6 @@ style_init (AppearanceData *data)
 {
   GtkSettings *settings;
   GtkWidget *w;
-  GtkAdjustment *adjustment;
   gchar *label;
   gint i;
 
@@ -891,10 +881,7 @@ style_init (AppearanceData *data)
 
 #ifdef HAVE_XCURSOR
   w = glade_xml_get_widget (data->xml, "cursor_size_scale");
-  GTK_RANGE (w)->round_digits = 0;
   g_signal_connect (w, "value-changed", (GCallback) cursor_size_scale_value_changed_cb, data);
-  adjustment = gtk_range_get_adjustment (GTK_RANGE (w));
-  g_object_set (adjustment, "page-size", 0.0, NULL);
 
   w = glade_xml_get_widget (data->xml, "cursor_size_small_label");
   label = g_strdup_printf ("<small><i>%s</i></small>", gtk_label_get_text (GTK_LABEL (w)));
@@ -924,6 +911,7 @@ style_init (AppearanceData *data)
   g_signal_connect (glade_xml_get_widget (data->xml, "gtk_themes_delete"), "clicked", (GCallback) gtk_theme_delete_cb, data);
   g_signal_connect (glade_xml_get_widget (data->xml, "window_themes_delete"), "clicked", (GCallback) window_theme_delete_cb, data);
   g_signal_connect (glade_xml_get_widget (data->xml, "icon_themes_delete"), "clicked", (GCallback) icon_theme_delete_cb, data);
+  g_signal_connect (glade_xml_get_widget (data->xml, "cursor_themes_delete"), "clicked", (GCallback) cursor_theme_delete_cb, data);
 
   gnome_theme_info_register_theme_change ((ThemeChangedCallback) changed_on_disk_cb, data);
 }
