@@ -16,6 +16,7 @@
 #include <libgnomeui/gnome-icon-theme.h>
 
 #include "theme-thumbnail.h"
+#include "gtkrc-utils.h"
 #include "capplet-util.h"
 
 static gint child_pid;
@@ -39,11 +40,11 @@ static ThemeThumbnailAsyncData async_data;
 
 /* Protocol */
 
-/* Our protocol is pretty simple.  The parent process will write four strings
- * (separated by a '\000') They are the widget theme, the wm theme, the icon
- * theme, and the font string.  Then, it will wait for the child to write back
- * the data.  The parent expects ICON_SIZE_WIDTH * ICON_SIZE_HEIGHT * 4 bytes of
- * information.  After that, the child is ready for the next theme to render.
+/* Our protocol is pretty simple.  The parent process will write several strings
+ * (separated by a '\000'). They are the widget theme, the wm theme, the icon
+ * theme, etc.  Then, it will wait for the child to write back the data.  The
+ * parent expects ICON_SIZE_WIDTH * ICON_SIZE_HEIGHT * 4 bytes of information.
+ * After that, the child is ready for the next theme to render.
  */
 
 enum
@@ -345,7 +346,9 @@ create_gtk_theme_pixbuf (ThemeThumbnailData *theme_thumbnail_data)
   gint width, height;
 
   settings = gtk_settings_get_default ();
-  g_object_set (settings, "gtk-theme-name", (char *) theme_thumbnail_data->control_theme_name->data, NULL);
+  g_object_set (settings, "gtk-theme-name", (char *) theme_thumbnail_data->control_theme_name->data,
+			  "gtk-color-scheme", (char *) theme_thumbnail_data->gtk_color_scheme->data,
+ 			  NULL);
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
@@ -932,12 +935,17 @@ generate_meta_theme_thumbnail (GnomeThemeMetaInfo *theme_info)
 GdkPixbuf *
 generate_gtk_theme_thumbnail (GnomeThemeInfo *theme_info)
 {
+  gchar *scheme;
+
+  scheme = gtkrc_get_color_scheme_for_theme (theme_info->name);
+
   return generate_theme_thumbnail (THUMBNAIL_TYPE_GTK,
                                    theme_info->name,
-                                   NULL,
+                                   scheme,
                                    NULL,
                                    NULL,
                                    NULL);
+  g_free (scheme);
 }
 
 GdkPixbuf *
@@ -1000,7 +1008,6 @@ generate_theme_thumbnail_async (gpointer            theme_info,
     return;
   }
 
-
   if (async_data.channel == NULL)
   {
     async_data.channel = g_io_channel_unix_new (pipe_from_factory_fd[0]);
@@ -1048,15 +1055,20 @@ generate_gtk_theme_thumbnail_async (GnomeThemeInfo *theme_info,
                                     gpointer            user_data,
                                     GDestroyNotify      destroy)
 {
+  gchar *scheme;
+
+  scheme = gtkrc_get_color_scheme_for_theme (theme_info->name);
+
   generate_theme_thumbnail_async (theme_info,
-                                         theme_info->name,
-                                         THUMBNAIL_TYPE_GTK,
-                                         theme_info->name,
-                                         NULL,
-                                         NULL,
-                                         NULL,
-                                         NULL,
-                                         func, user_data, destroy);
+                                  theme_info->name,
+                                  THUMBNAIL_TYPE_GTK,
+                                  theme_info->name,
+                                  scheme,
+                                  NULL,
+                                  NULL,
+                                  NULL,
+                                  func, user_data, destroy);
+  g_free (scheme);
 }
 
 void
