@@ -1,5 +1,5 @@
 /*
- * Copyright © 2001 Red Hat, Inc.
+ * Copyright © 2001, 2007 Red Hat, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -15,17 +15,16 @@
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL RED HAT
  * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
- * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
+ * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * Authors:  Owen Taylor, Havoc Pennington
+ * Authors:  Owen Taylor, Havoc Pennington, Ray Strode, Rodrigo Moya
  */
 #include <config.h>
 
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
-
 #include <gconf/gconf.h>
 #include <libgnome/gnome-init.h>
 #include <libgnomeui/gnome-ui-init.h>
@@ -33,51 +32,31 @@
 #include "xsettings-manager.h"
 #include "gnome-settings-daemon.h"
 #include "gnome-settings-module.h"
-#include "gnome-settings-xmodmap.h"
-
-/*#include "gnome-settings-disk.h"*/
-#include "gnome-settings-keyboard-xkb.h"
-#include "gnome-settings-keyboard.h"
-#include "gnome-settings-accessibility-keyboard.h"
-#include "gnome-settings-keybindings.h"
-#include "gnome-settings-multimedia-keys.h"
-#include "gnome-settings-xrdb.h"
 
 struct _GnomeSettingsDaemonPrivate {
-	GHashTable *loaded_modules;
+	GSList *loaded_modules;
+	GObject *dbus_service;
 };
 
+GType gnome_settings_module_accessibility_keyboard_get_type (void);
 GType gnome_settings_module_background_get_type (void);
 GType gnome_settings_module_clipboard_get_type (void);
 GType gnome_settings_module_default_editor_get_type (void);
 GType gnome_settings_module_font_get_type (void);
 GType gnome_settings_module_gtk1_get_type (void);
+GType gnome_settings_module_keybindings_get_type (void);
+GType gnome_settings_module_keyboard_get_type (void);
 GType gnome_settings_module_mouse_get_type (void);
+GType gnome_settings_module_multimedia_keys_get_type (void);
 GType gnome_settings_module_screensaver_get_type (void);
 GType gnome_settings_module_sound_get_type (void);
 GType gnome_settings_module_typing_break_get_type (void);
+GType gnome_settings_module_xrdb_get_type (void);
 GType gnome_settings_module_xsettings_get_type (void);
 
-static GObjectClass *parent_class = NULL;
+static GObject *parent_class = NULL;
 XSettingsManager **managers = NULL;
 
-static void
-debug_warning (const char *msg, ...)
-{
-	va_list args;
-	gchar *str;
-	GtkWidget *dialog;
-
-	va_start (args, msg);
-	str = g_strdup_vprintf (msg, args);
-	va_end (args);
-
-	dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE, str);
-	gnome_settings_delayed_show_dialog (dialog);
-
-	g_free (str);
-}
-
 static void
 terminate_cb (void *data)
 {
@@ -85,12 +64,12 @@ terminate_cb (void *data)
 
 	if (*terminated)
 		return;
-  
+
 	*terminated = TRUE;
 	gtk_main_quit ();
 }
 
-static GdkFilterReturn 
+static GdkFilterReturn
 manager_event_filter (GdkXEvent *xevent,
 		      GdkEvent  *event,
 		      gpointer   data)
@@ -105,50 +84,48 @@ manager_event_filter (GdkXEvent *xevent,
 		return GDK_FILTER_CONTINUE;
 }
 
-static void
-free_modules_list (gpointer data)
-{
-	GList *l = (GList *) data;
-
-	while (l) {
-		g_object_unref (G_OBJECT (l->data));
-		l = g_list_remove (l, l);
-	}
-}
-
 static void
 initialize_modules (GnomeSettingsDaemon *daemon, GnomeSettingsModuleRunlevel runlevel)
 {
-	GList *l, *module_list;
 	GConfClient *client;
+	GSList *l;
 
 	client = gnome_settings_get_config_client ();
 
-	module_list = g_hash_table_lookup (daemon->priv->loaded_modules, &runlevel);
-	for (l = module_list; l != NULL; l = l->next) {
-		
-		gnome_settings_module_initialize (GNOME_SETTINGS_MODULE (l->data), client);
+	for (l = daemon->priv->loaded_modules; l != NULL; l = l->next) {
+		GnomeSettingsModule *module = (GnomeSettingsModule *) l->data;
+
+		if (gnome_settings_module_get_runlevel (module) == runlevel &&
+		    !gnome_settings_module_initialize (module, client))
+			g_warning ("Module %s could not be initialized", G_OBJECT_TYPE_NAME (module));
 	}
 }
 
 static void
 start_modules (GnomeSettingsDaemon *daemon, GnomeSettingsModuleRunlevel runlevel)
 {
-	GList *l, *module_list;
+	GSList *l;
 
-	module_list = g_hash_table_lookup (daemon->priv->loaded_modules, &runlevel);
-	for (l = module_list; l != NULL; l = l->next)
-		gnome_settings_module_start (GNOME_SETTINGS_MODULE (l->data));
+	for (l = daemon->priv->loaded_modules; l != NULL; l = l->next) {
+		GnomeSettingsModule *module = (GnomeSettingsModule *) l->data;
+
+		if (gnome_settings_module_get_runlevel (module) == runlevel &&
+		    !gnome_settings_module_start (module))
+			g_warning ("Module %s could not be started", G_OBJECT_TYPE_NAME (module));
+	}
 }
 
 static void
 stop_modules (GnomeSettingsDaemon *daemon, GnomeSettingsModuleRunlevel runlevel)
 {
-	GList *l, *module_list;
+	GSList *l;
 
-	module_list = g_hash_table_lookup (daemon->priv->loaded_modules, &runlevel);
-	for (l = module_list; l != NULL; l = l->next)
-		gnome_settings_module_stop (GNOME_SETTINGS_MODULE (l->data));
+	for (l = daemon->priv->loaded_modules; l != NULL; l = l->next) {
+		GnomeSettingsModule *module = (GnomeSettingsModule *) l->data;
+
+		if (gnome_settings_module_get_runlevel (module) == runlevel)
+			gnome_settings_module_stop (module);
+	}
 }
 
 static void
@@ -171,10 +148,12 @@ finalize (GObject *object)
 		stop_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_GNOME_SETTINGS);
 		stop_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_XSETTINGS);
 
-		g_hash_table_destroy (daemon->priv->loaded_modules);
+		g_slist_foreach (daemon->priv->loaded_modules, (GFunc) g_object_unref, NULL);
+		g_slist_free (daemon->priv->loaded_modules);
 		daemon->priv->loaded_modules = NULL;
 	}
 
+	g_object_unref (daemon->priv->dbus_service);
 	g_free (daemon->priv);
 	daemon->priv = NULL;
 
@@ -199,112 +178,75 @@ gnome_settings_daemon_init (GnomeSettingsDaemon *settings)
 	GType *module_types;
 	guint n_children;
 
-	settings->priv = g_new (GnomeSettingsDaemonPrivate, 1);
+	settings->priv = g_new0 (GnomeSettingsDaemonPrivate, 1);
 
 	/* register all internal modules types */
-	if (!gnome_settings_module_background_get_type ()
+	if (!gnome_settings_module_accessibility_keyboard_get_type ()
+	    || !gnome_settings_module_background_get_type ()
 	    || !gnome_settings_module_clipboard_get_type ()
 	    || !gnome_settings_module_default_editor_get_type ()
 	    || !gnome_settings_module_font_get_type ()
 	    || !gnome_settings_module_gtk1_get_type ()
+	    || !gnome_settings_module_keybindings_get_type ()
+	    || !gnome_settings_module_keyboard_get_type ()
 	    || !gnome_settings_module_mouse_get_type ()
+	    || !gnome_settings_module_multimedia_keys_get_type ()
 	    || !gnome_settings_module_screensaver_get_type ()
 	    || !gnome_settings_module_sound_get_type ()
 	    || !gnome_settings_module_typing_break_get_type ()
+	    || !gnome_settings_module_xrdb_get_type ()
 	    || !gnome_settings_module_xsettings_get_type ())
 		return;
 
-	/* create hash table for loaded modules */
-	settings->priv->loaded_modules = g_hash_table_new_full (g_int_hash, g_int_equal, NULL, free_modules_list);
-
 	module_types = g_type_children (GNOME_SETTINGS_TYPE_MODULE, &n_children);
 	if (module_types) {
 		guint i;
 
 		for (i = 0; i < n_children; i++) {
 			GObject *module;
-			GnomeSettingsModuleRunlevel runlevel;
-			GList *module_list;
 
 			module = g_object_new (module_types[i], NULL);
 			if (!module)
 				continue;
 
-			runlevel = gnome_settings_module_get_runlevel (GNOME_SETTINGS_MODULE (module));
-			module_list = g_hash_table_lookup (settings->priv->loaded_modules, &runlevel);
-			if (module_list)
-				module_list = g_list_append (module_list, module);
-			else {
-				module_list = g_list_append (NULL, module);
-				g_hash_table_insert (settings->priv->loaded_modules, &runlevel, module_list);
-			}
+			settings->priv->loaded_modules =
+				g_slist_append (settings->priv->loaded_modules, module);
 		}
 
 		g_free (module_types);
 	}
+
+	/* create DBus server */
+	settings->priv->dbus_service = g_object_new (gnome_settings_server_get_type (), NULL);
 }
 
 G_DEFINE_TYPE (GnomeSettingsDaemon, gnome_settings_daemon, G_TYPE_OBJECT)
 
-GObject *
-gnome_settings_daemon_new (void)
+static gboolean
+start_modules_idle_cb (gpointer user_data)
+{
+	GnomeSettingsDaemon *daemon = user_data;
+
+	/* start all modules */
+	start_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_XSETTINGS);
+	start_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_GNOME_SETTINGS);
+	start_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_CORE_SERVICES);
+	start_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_SERVICES);
+
+	return FALSE;
+}
+
+static gboolean
+init_modules_idle_cb (gpointer user_data)
 {
-	gboolean terminated = FALSE;
-	GConfClient *client;
-	GnomeSettingsDaemon *daemon;
-	GdkDisplay *display;
-	GObject *dbusServer;
 	int i;
 	int n_screens;
+	GdkDisplay *display;
+	GnomeSettingsDaemon *daemon = user_data;
 
 	display = gdk_display_get_default ();
 	n_screens = gdk_display_get_n_screens (display);
 
-	daemon = g_object_new (gnome_settings_daemon_get_type (), NULL);
-
-	if (xsettings_manager_check_running (
-		    gdk_x11_display_get_xdisplay (display),
-		    gdk_screen_get_number (gdk_screen_get_default ()))) {
-		fprintf (stderr, "You can only run one xsettings manager at a time; exiting\n");
-		return NULL;
-	}
-
-  
-	if (!terminated) {
-		managers = g_new (XSettingsManager *, n_screens + 1);
-
-		for (i = 0; i < n_screens; i++) {
-			GdkScreen *screen;
-
-			screen = gdk_display_get_screen (display, i);
-
-			managers [i] = xsettings_manager_new (
-				gdk_x11_display_get_xdisplay (display),
-				gdk_screen_get_number (screen),
-				terminate_cb, &terminated);
-			if (!managers [i]) {
-				fprintf (stderr, "Could not create xsettings manager for screen %d!\n", i);
-				return NULL;
-			}
-		}
-
-		g_assert (i == n_screens);
-		managers [i] = NULL;
-	}
-
-	client = gnome_settings_get_config_client ();
-
-        /*  gnome_settings_disk_init (client);*/
-	/* Essential - xkb initialization should happen before */
-	gnome_settings_keyboard_xkb_set_post_activation_callback ((PostActivationCallback) gnome_settings_load_modmap_files, NULL);
-	gnome_settings_keyboard_xkb_init (client);
-	gnome_settings_keyboard_init (client);
-	gnome_settings_multimedia_keys_init (client);
-        /* */
-	gnome_settings_accessibility_keyboard_init (client);
-	gnome_settings_keybindings_init (client);
-	gnome_settings_xrdb_init (client);
-
 	/* load all modules */
 	initialize_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_XSETTINGS);
 	initialize_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_GNOME_SETTINGS);
@@ -320,24 +262,54 @@ gnome_settings_daemon_new (void)
 			manager_event_filter, GINT_TO_POINTER (i));
 	}
 
-	/*  gnome_settings_disk_load (client);*/
-	/* Essential - xkb initialization should happen before */
-	gnome_settings_keyboard_xkb_load (client);
-	gnome_settings_keyboard_load (client);
-	gnome_settings_multimedia_keys_load (client);
-	/* */
-	gnome_settings_accessibility_keyboard_load (client);
-	gnome_settings_keybindings_load (client);
-	gnome_settings_xrdb_load (client);
+	g_idle_add ((GSourceFunc) start_modules_idle_cb, daemon);
 
-	/* start all modules */
-	start_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_XSETTINGS);
-	start_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_GNOME_SETTINGS);
-	start_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_CORE_SERVICES);
-	start_modules (daemon, GNOME_SETTINGS_MODULE_RUNLEVEL_SERVICES);
+	return FALSE;
+}
 
-	/* create DBus server */
-	dbusServer = g_object_new (gnome_settings_server_get_type (), NULL);
+GObject *
+gnome_settings_daemon_new (void)
+{
+	gboolean terminated = FALSE;
+	GnomeSettingsDaemon *daemon;
+	GdkDisplay *display;
+	int i;
+	int n_screens;
+
+	display = gdk_display_get_default ();
+	n_screens = gdk_display_get_n_screens (display);
+
+	daemon = g_object_new (gnome_settings_daemon_get_type (), NULL);
+
+	if (xsettings_manager_check_running (
+		    gdk_x11_display_get_xdisplay (display),
+		    gdk_screen_get_number (gdk_screen_get_default ()))) {
+		g_error ("You can only run one xsettings manager at a time; exiting\n");
+		return NULL;
+	}
+
+	if (!terminated) {
+		managers = g_new (XSettingsManager *, n_screens + 1);
+
+		for (i = 0; i < n_screens; i++) {
+			GdkScreen *screen;
+
+			screen = gdk_display_get_screen (display, i);
+
+			managers [i] = xsettings_manager_new (
+				gdk_x11_display_get_xdisplay (display),
+				gdk_screen_get_number (screen),
+				terminate_cb, &terminated);
+			if (!managers [i]) {
+				g_error ("Could not create xsettings manager for screen %d!\n", i);
+				return NULL;
+			}
+		}
+
+		managers [i] = NULL;
+	}
+
+	g_idle_add ((GSourceFunc) init_modules_idle_cb, daemon);
 
 	return G_OBJECT (daemon);
 }
