@@ -646,7 +646,9 @@ do_sound_action (Acme *acme, int type)
 		break;
 	case VOLUME_UP_KEY:
 		if (muted) {
-			acme_volume_mute_toggle(acme->volobj);
+			if (vol == 0)
+				acme_volume_set_volume (acme->volobj, vol + vol_step);
+			acme_volume_mute_toggle (acme->volobj);
 		} else {
 			acme_volume_set_volume (acme->volobj, vol + vol_step);
 		}
@@ -725,6 +727,9 @@ do_action (Acme *acme, int type)
 	case MEDIA_KEY:
 		do_media_action (acme);
 		break;
+	case CALCULATOR_KEY:
+		execute (acme, "gcalctool", FALSE, FALSE);
+		break;
 	case PLAY_KEY:
 		return do_multimedia_player_action (acme, "Play");
 		break;
