@@ -142,7 +142,7 @@ static gchar *find_package_data_file (const gchar *filename);
 
 
 GType
-bookmark_agent_get_type ()
+bookmark_agent_get_type (void)
 {
 	static GType g_define_type_id = 0;
 
