@@ -559,7 +559,7 @@ G_LOCK_DEFINE_STATIC(monitor_list);
 static GList *monitor_list = NULL;
 
 static void
-invoke_monitors()
+invoke_monitors(void)
 {
     GList *tmp;
 
@@ -574,11 +574,10 @@ invoke_monitors()
 }
 
 static void
-theme_changed_callback(GnomeThemeType       type,
-                       gpointer             theme,
-                       GnomeThemeChangeType change_type,
-                       GnomeThemeElement    element,
-                       gpointer             data)
+theme_changed_callback(GnomeThemeCommonInfo *theme,
+                       GnomeThemeChangeType  change_type,
+                       GnomeThemeElement     element,
+                       gpointer              data)
 {
   invoke_monitors();
 }
